% Global variables to share with other files.
global u;
global bExit bAuto bLineFollowing;
global scale offsetx offsety;
global L l alpha alphaf beta;

button = questdlg('Controls are z,s,q,d keys, p to switch autonomous mode, zoom with +,-, move camera with i,j,k,l and exit with ESC. Ready?', 'Ready?');
if (strcmp(button, 'Yes') == 0)
    return;
end

% Create a figure that will use the function keycontrol() when a key is
% pressed.
fig = figure('Position',[200 200 400 400],'KeyPressFcn','keycontrol','Name','Motorboat simu','NumberTitle','off');
% Force the figure to have input focus (required to capture keys).
set(fig,'WindowStyle','Modal');
axis('off');

bExit = 0; bAuto = 0; bLineFollowing = 0;
scale = 5; offsetx = 0; offsety = -50;

L = 1.22; % Length of the hull (in m).
l = 0.35; % Width of the hull (in m).
alpha = 11.5; % Coefficient to set depending on boat speed for a specific u(1) input.
alphaf = 1.3; % Coefficient to set depending on damping.
beta = 0.4; % Coefficient (in rad) to set depending on max jet angle.

x = [0;-50;0;0;0];
u = [0;0];
dt = 0.05; t = 0;

targets = [3.895810 -60.527813;
-2.875225 -80.404199;
-7.351641 -82.607217;
-9.548130 -78.117508;
-2.777095 -58.241114;
-4.973585 -53.751403;
-9.450000 -55.954423;
-16.221035 -75.830824;
-20.697450 -78.033853;
-22.893939 -73.544149;
-16.122905 -53.667740;
-18.319395 -49.178034;
-22.795810 -51.381065;
-29.566844 -71.257481;
-34.043259 -73.460520;
-36.239749 -68.970821;
-29.468715 -49.094397;
-31.665204 -44.604697;
-36.141619 -46.807738;
-42.912653 -66.684170;
-47.389068 -68.887220;
-49.585557 -64.397526;
-42.814524 -44.521087;
-45.011013 -40.031392;
-49.487428 -42.234444;
-56.258461 -62.110891;
-60.734876 -64.313951;
-62.931365 -59.824263;
-56.160332 -39.947809;
-58.356822 -35.458119;
-62.833236 -37.661181;
-60.636747 -42.150869;
6.092300 -65.017528;
8.288789 -69.507244;
3.812374 -71.710254;
-62.916672 -48.843621;
-67.393087 -51.046686;
-65.196598 -55.536373;
1.532447 -78.402980;
3.728937 -82.892694;
-0.747479 -85.095708;
-67.476523 -62.229126;
];
i = 1;
target = [targets(i, 1) targets(i, 2)];
X=[];

while (bExit == 0)
    % Simulation part.
    x = x+f(x,u)*dt;
    y = g(x,u);
    
    % Autonomous control part to go to waypoints. Could be replaced by line
    % following...
    if (bAuto == 1)
        theta_w = atan2(target(2)-y(2),target(1)-y(1));    
        if (cos(theta_w-y(3)) > 0.5)
            u(2) = sin(theta_w-y(3));
        else
            u(2) = sign(sin(theta_w-y(3)));
        end
        u(1) = 0.17;
    end
   
    clf; hold on;
    axis([-scale+offsetx,scale+offsetx,-scale+offsety,scale+offsety]); axis square;
    draw(x,u); % Real position of the simulated robot.
    
    X=[X;x(1) x(2)];
    
    % Draw current waypoint and change it when we are close.
    plot(target(1),target(2),'--rs','LineWidth',5,'MarkerSize',5);    
    if ((target(1)-2.5 < y(1))&&(y(1) < target(1)+2.5)&&(target(2)-2.5 < y(2))&&(y(2) < target(2)+2.5))
        i = i+1;
        if (i > length(targets))
            break;
        end
        target = [targets(i, 1) targets(i, 2)];
    end

    % Wait a little bit.
    pause(0.02); % <dt because there are also computations delays...
    t = t+dt;
end

t

csvwrite('log.csv',X)

close(fig);
